"""
pten.wwcontact
~~~~~~~~~~~~

This module implements the Contact class.

"""

from .keys import Keys
from .wwapi import CorpApi, CORP_API_TYPE


def encode_url(url):
    import urllib.parse

    return urllib.parse.quote(url, safe="")


class Contact:
    def __init__(
        self,
        keys_filepath="pten_keys.ini",
        contact_sync_secret=None,
        keys: Keys = None,
        **kwargs,
    ):
        self.keys = keys if keys else Keys(keys_filepath)
        if contact_sync_secret is None:
            contact_sync_secret = self.keys.get_contact_sync_secret()

        self.api = CorpApi(keys_filepath, corpsecret=contact_sync_secret)

    def create_user(self, userid, name, **kwargs):
        data = {}
        data["userid"] = userid
        data["name"] = name
        data.update(kwargs)

        return self.api.http_call(CORP_API_TYPE["USER_CREATE"], data)

    def get_user(self, userid):
        return self.api.http_call(CORP_API_TYPE["USER_GET"], {"userid": userid})

    def update_user(self, userid, **kwargs):
        """
        仅通讯录同步助手或第三方通讯录应用可调用
        """
        data = {}
        data["userid"] = userid
        data.update(kwargs)

        return self.api.http_call(CORP_API_TYPE["USER_UPDATE"], data)

    def delete_user(self, userid):
        """
        仅通讯录同步助手或第三方通讯录应用可调用
        """
        return self.api.http_call(CORP_API_TYPE["USER_DELETE"], {"userid": userid})

    def batch_delete_user(self, useridlist):
        """
        仅通讯录同步助手或第三方通讯录应用可调用
        """
        data = {"useridlist": useridlist}
        return self.api.http_call(CORP_API_TYPE["USER_BATCH_DELETE"], data)

    def get_user_simple_list(self, department_id):
        """
        获取部门成员
        """
        data = {"department_id": department_id}
        return self.api.http_call(CORP_API_TYPE["USER_SIMPLE_LIST"], data)

    def get_user_list(self, department_id):
        """
        获取部门成员详情
        """
        data = {"department_id": department_id}
        return self.api.http_call(CORP_API_TYPE["USER_LIST"], data)

    def convert_userid_to_openid(self, userid):
        return self.api.http_call(CORP_API_TYPE["USERID_TO_OPENID"], {"userid": userid})

    def convert_openid_to_userid(self, openid):
        return self.api.http_call(CORP_API_TYPE["OPENID_TO_USERID"], {"openid": openid})

    def invite_users(self, userlist):
        return self.api.http_call(CORP_API_TYPE["BATCH_INVITE"], {"user": userlist})

    def get_join_qrcode(self, size_type):
        """
        须拥有通讯录的管理权限，使用通讯录同步的Secret。
        """
        data = {"size_type": size_type}
        return self.api.http_call(CORP_API_TYPE["GET_JOIN_QRCODE"], data)

    def get_userid_by_mobile(self, mobile):
        data = {"mobile": mobile}
        return self.api.http_call(CORP_API_TYPE["GET_USERID_BY_MOBILE"], data)

    def get_userid_by_email(self, email):
        return self.api.http_call(
            CORP_API_TYPE["GET_USERID_BY_EMAIL"], {"email": email}
        )

    def get_user_id_list(self):
        """
        仅支持通过“通讯录同步secret”调用。
        """
        return self.api.http_call(CORP_API_TYPE["USER_ID_LIST"])

    def create_department(self, name, parentid, **kwargs):
        """
        第三方仅通讯录应用可以调用。
        """
        data = {"name": name, "parentid": parentid}
        data.update(kwargs)

        return self.api.http_call(CORP_API_TYPE["DEPARTMENT_CREATE"], data)

    def update_department(self, id, **kwargs):
        """
        应用须拥有指定部门的管理权限。如若要移动部门，需要有新父部门的管理权限。
        第三方仅通讯录应用可以调用。
        """
        data = {"id": id}
        data.update(kwargs)

        return self.api.http_call(CORP_API_TYPE["DEPARTMENT_UPDATE"], data)

    def delete_department(self, id):
        """
        应用须拥有指定部门的管理权限。
        第三方仅通讯录应用可以调用。
        """
        return self.api.http_call(CORP_API_TYPE["DEPARTMENT_DELETE"], {"id": id})

    def get_department_list(self):
        """
        从2022年8月15日10点开始, “企业管理后台 - 管理工具 - 通讯录同步”的新增IP将不能再调用此接口
        企业可通过「获取部门ID列表」接口获取部门ID列表。
        """
        return self.api.http_call(CORP_API_TYPE["DEPARTMENT_LIST"])

    def get_department_simplelist(self):
        return self.api.http_call(CORP_API_TYPE["DEPARTMENT_SIMPLE_LIST"])

    def get_department_detail(self, id):
        """
        从2022年8月15日10点开始, “企业管理后台 - 管理工具 - 通讯录同步”的新增IP将不能再调用此接口
        企业可通过「获取部门ID列表」接口获取部门ID列表。
        """
        return self.api.http_call(CORP_API_TYPE["DEPARTMENT_GET"], {"id": id})

    def create_tag(self, tagname, tagid=None):
        data = {"tagname": tagname}
        if tagid:
            data["tagid"] = tagid

        return self.api.http_call(CORP_API_TYPE["TAG_CREATE"], data)

    def udpate_tag(self, tagname, tagid):
        """
        调用的应用必须是指定标签的创建者。
        """
        data = {"tagid": tagid, "tagname": tagname}
        return self.api.http_call(CORP_API_TYPE["TAG_UPDATE"], data)

    def delete_tag(self, tagid):
        """
        调用的应用必须是指定标签的创建者。
        """
        return self.api.http_call(CORP_API_TYPE["TAG_DELETE"], {"tagid": tagid})

    def get_tag_user(self, tagid):
        """ "
        无限制，但返回列表仅包含应用可见范围的成员；第三方可获取自己创建的标签及应用可见范围内的标签详情
        """
        return self.api.http_call(CORP_API_TYPE["TAG_GET_USER"], {"tagid": tagid})

    def add_tag_user(self, tagid, userlist=None, partylist=None):
        """
        调用的应用必须是指定标签的创建者；成员属于应用的可见范围。
        注意，每个标签下部门数和人员数总和不能超过3万个。
        """
        data = {"tagid": tagid}
        if userlist:
            data["userlist"] = userlist
        if partylist:
            data["partylist"] = partylist

        if not userlist and not partylist:
            raise ValueError("userlist or partylist is required")
        if (len(userlist) > 1000) or (len(partylist) > 100):
            raise ValueError("(len(userlist) > 1000) or (len(partylist) > 100)")

        return self.api.http_call(CORP_API_TYPE["TAG_ADD_USER"], data)

    def delete_tag_user(self, tagid, userlist=None, partylist=None):
        """
        调用的应用必须是指定标签的创建者；成员属于应用的可见范围。
        """
        data = {"tagid": tagid}
        if userlist:
            data["userlist"] = userlist
        if partylist:
            data["partylist"] = partylist

        if not userlist and not partylist:
            raise ValueError("userlist or partylist is required")
        if (len(userlist) > 1000) or (len(partylist) > 100):
            raise ValueError("(len(userlist) > 1000) or (len(partylist) > 100)")

        return self.api.http_call(CORP_API_TYPE["TAG_DELETE_USER"], data)

    def get_tag_list(self):
        """
        自建应用或通讯同步助手可以获取所有标签列表；第三方应用仅可获取自己创建的标签。
        """
        return self.api.http_call(CORP_API_TYPE["TAG_GET_LIST"])

    def batch_sync_users(self, media_id, **kwargs):
        """
        须拥有通讯录的写权限。
        """
        data = {"media_id": media_id}
        data.update(kwargs)

        return self.api.http_call(CORP_API_TYPE["BATCH_SYNC_USERS"], data)

    def batch_replace_users(self, media_id, **kwargs):
        """
        须拥有通讯录的写权限。
        """
        data = {"media_id": media_id}
        data.update(kwargs)

        return self.api.http_call(CORP_API_TYPE["BATCH_REPLACE_USERS"], data)

    def batch_replace_departments(self, media_id, **kwargs):
        """
        须拥有通讯录的写权限。
        """
        data = {"media_id": media_id}
        data.update(kwargs)

        return self.api.http_call(CORP_API_TYPE["BATCH_REPLACE_PARTY"], data)

    def batch_get_job_result(self, jobid):
        """
        只能查询已经提交过的历史任务。
        """
        data = {"jobid": jobid}
        return self.api.http_call(CORP_API_TYPE["BATCH_JOB_GET_RESULT"], data)

    def export_simple_user(self, encoding_aeskey, block_size=None):
        """
        仅会返回有权限的人员列表
        """
        data = {"encoding_aeskey": encoding_aeskey}
        if block_size:
            data["block_size"] = block_size

        return self.api.http_call(CORP_API_TYPE["EXPORT_SIMPLE_USER"], data)

    def export_user(self, encoding_aeskey, block_size=None):
        """
        仅会返回有权限的人员列表
        """
        data = {"encoding_aeskey": encoding_aeskey}
        if block_size:
            data["block_size"] = block_size

        return self.api.http_call(CORP_API_TYPE["EXPORT_USER"], data)

    def export_department(self, encoding_aeskey, block_size=None):
        """
        仅返回有权限的部门列表
        从2022年8月15日10点开始, “企业管理后台 - 管理工具 - 通讯录同步”的新增IP将不能再调用此接口
        企业可通过「获取成员ID列表」和「获取部门ID列表」接口获取userid和部门ID列表。
        """
        data = {"encoding_aeskey": encoding_aeskey}
        if block_size:
            data["block_size"] = block_size

        return self.api.http_call(CORP_API_TYPE["EXPORT_DEPARTMENT"], data)

    def export_tag_user(self, tagid, encoding_aeskey, block_size=None):
        """
        要求对标签有读取权限
        """
        data = {"tagid": tagid, "encoding_aeskey": encoding_aeskey}
        if block_size:
            data["block_size"] = block_size

        return self.api.http_call(CORP_API_TYPE["EXPORT_TAG_USER"], data)

    def export_get_result(self, jobid):
        """
        获取任务结果的调用身份需要与提交任务的一致
        """
        data = {"jobid": jobid}
        return self.api.http_call(CORP_API_TYPE["EXPORT_GET_RESULT"], data)

    def auth_get_user_info(self, code):
        """
        跳转的域名须完全匹配access_token对应应用的可信域名，否则会返回50001错误。
        """
        return self.api.http_call(CORP_API_TYPE["AUTH_GET_USER_INFO"], {"code": code})

    def auth_get_user_detail(self, user_ticket):
        """
        跳转的域名须完全匹配access_token对应应用的可信域名，否则会返回50001错误。
        """
        data = {"user_ticket": user_ticket}
        return self.api.http_call(CORP_API_TYPE["AUTH_GET_USER_DETAIL"], data)



"""
pten.wwmessager
~~~~~~~~~~~~

This module implements the Messager class.

Many codes are from “corpwechatbot"

"""

from . import logger
from .keys import Keys
from .wwapi import BotApi, BOT_API_TYPE, CorpApi, CORP_API_TYPE
import base64
from hashlib import md5
from pathlib import Path
from queue import Queue
import time
from typing import Optional


def reply_text(decrypt_data, content):
    sRespData = """
<xml>
   <ToUserName>{to_username}</ToUserName>
   <FromUserName>{from_username}</FromUserName> 
   <CreateTime>{create_time}</CreateTime>
   <MsgType>text</MsgType>
   <Content>{content}</Content>
</xml>
""".format(
        to_username=decrypt_data["ToUserName"],
        from_username=decrypt_data["FromUserName"],
        create_time=decrypt_data["CreateTime"],
        content=content,
    )

    return sRespData


def is_image(image_path: str):
    p_image = Path(image_path)
    if not p_image.is_file():
        return False
    if p_image.suffix != ".jpg" and p_image.suffix != ".png":
        return False
    if p_image.stat().st_size > 2 * 1024 * 1024 or p_image.stat().st_size <= 5:
        # image no more than 2M
        return False
    return True


def is_voice(voice_path: str):
    p_voice = Path(voice_path)
    if not p_voice.is_file():
        return False
    if p_voice.suffix != ".amr":
        return False
    if p_voice.stat().st_size > 2 * 1024 * 1024 or p_voice.stat().st_size <= 5:
        # voice no more than 2M
        return False
    return True


def is_video(video_path: str):
    p_video = Path(video_path)
    if not p_video.is_file():
        return False
    if p_video.suffix != ".mp4":
        return False
    if p_video.stat().st_size > 10 * 1024 * 1024 or p_video.stat().st_size <= 5:
        # video no more than 10M
        return False
    return True


def is_file(file_path: str):
    p_file = Path(file_path)
    if not p_file.is_file():
        return False
    if p_file.stat().st_size > 20 * 1024 * 1024 or p_file.stat().st_size <= 5:
        # file no more than 2M
        return False
    return True


class MsgSender:
    """
    The parent class of all the notify classes
    """

    def __init__(self, keys_filepath="pten_keys.ini", keys: Keys = None, **kwargs):
        self.keys = keys if keys else Keys(keys_filepath)
        self.errmsgs = {
            "image_error": "图片文件不合法",
            "text_error": "文本消息不合法",
            "news_error": "图文消息内容不合法",
            "markdown_error": "markdown内容不合法",
            "voice_error": "语音文件不合法",
            "video_error": "视频文件不合法",
            "file_error": "文件不合法",
            "card_error": "卡片消息不合法",
            "media_error": "media_id获取失败",
            "mpnews_error": "mp图文消息不合法",
            "taskcard_error": "任务卡片消息不合法",
            "create_chat_error": "群聊创建失败，人数不能低于2",
        }

    def _get_media_id(self, media_type: str, p_media: Path):
        """
        获取media id，微信要求文件先上传到其后端服务器，再获取相应media id
        :param media_type:
        :param p_media:
        :return:
        """
        raise NotImplementedError

    def _send(
        self,
        msg_type: str = "",
        data: dict = {},
        media_path: Optional[str] = "",
        **kwargs,
    ):
        """
        :param msg_type:
        :param data:
        :param media_path:
        :param kwargs:
        :return:
        """

    def send_text(self, *args, **kwargs):
        """
        send text message
        :return:
        """
        raise NotImplementedError

    def send_markdown(self, *args, **kwargs):
        """
        send markdown message
        :return:
        """
        raise NotImplementedError

    def send_image(self, *args, **kwargs):
        """
        send image message
        :return:
        """
        raise NotImplementedError

    def send_voice(self, *args, **kwargs):
        """
        发送语音消息
        """
        raise NotImplementedError

    def send_video(self, *args, **kwargs):
        """
        发送视频消息
        """
        raise NotImplementedError

    def send_news(self, *args, **kwargs):
        """
        send news
        :return:
        """
        raise NotImplementedError

    def send_file(self, *args, **kwargs):
        """
        send file
        :return:
        """
        raise NotImplementedError

    def send_mpnews(self, *args, **kwargs):
        """
        发送mpnews图文消息
        :param args:
        :param kwargs:
        :return:
        """
        raise NotImplementedError

    def send_card(self, *args, **kwargs):
        """
        发送卡片消息
        """
        raise NotImplementedError

    def send_miniprogram_notice(self, *args, **kwargs):
        raise NotImplementedError

    def send_template_card(self, *args, **kwargs):
        """
        发送模板卡片消息
        """
        raise NotImplementedError


class BotMsgSender(MsgSender):
    """
    企业微信机器人，支持文本、markdown、图片、图文、文件、语音类型数据的发送
    """

    def __init__(self, keys_filepath="pten_keys.ini", keys: Keys = None, **kwargs):
        super().__init__(keys_filepath, keys=keys, **kwargs)
        self.api = BotApi(keys_filepath, keys=keys)
        self.queue = Queue(20)  # 机器人消息频率限制为每分钟不超过20条消息

    def _get_media_id(self, media_type: str, p_media: Path):
        if media_type not in ["voice", "file"]:
            raise ValueError("bot media_type can only be voice or file")

        data = {"file": p_media.open("rb")}
        response = self.api.http_call(
            BOT_API_TYPE["WEBHOOK_MEDIA_UPLOAD"],
            {"type": media_type, "files": data},
        )

        return response

    def _send(
        self,
        msg_type: str = "",
        data: dict = {},
        media_path: Optional[str] = "",
        **kwargs,
    ):
        """
        :param msg_type:
        :param data:
        :param media_path:
        :return:
        """
        data["msgtype"] = msg_type

        now = time.time()
        self.queue.put(now)
        if self.queue.full():
            # 限制每分钟20条消息，超限则进行睡眠等待
            interval_time = now - self.queue.get()
            if interval_time < 60:
                sleep_time = int(60 - interval_time) + 1
                logger.debug(f"机器人每分钟限制20条消息，需等待 {sleep_time} s")
                time.sleep(sleep_time)

        return self.api.http_call(BOT_API_TYPE["WEBHOOK_SEND"], data)

    def send_text(self, content, mentioned_list=[], mentioned_mobile_list=[]):
        """
        发送文本消息，
        :param content: 文本内容，最长不能超过2048字节，utf-8编码
        :param mentioned_list: userid列表，提醒群众某个成员，userid通过企业通讯录查看，'@all'则提醒所有人
        :param mentioned_mobile_list: 手机号列表，提醒手机号对应的成员，'@all'代表所有人，当不清楚userid时可替换
        :return: 消息发送结果
        """
        if not content:
            logger.error(self.errmsgs["text_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["text_error"]}
        data = {
            "text": {
                "content": content,
                "mentioned_list": mentioned_list,
                "mentioned_mobile_list": mentioned_mobile_list,
            }
        }
        return self._send(msg_type="text", data=data)

    def send_markdown(self, content=""):
        """
        发送markdown类型数据，支持markdown语法
        :param content: mardkown原始数据或markdown文件路径
        :return: 消息发送结果
        """
        if not content:
            logger.error(self.errmsgs["markdown_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["markdown_error"]}
        if content.endswith(".md"):
            try:
                content = Path(content).read_text()
            except (OSError, FileNotFoundError):
                logger.warning(
                    "你可能正在尝试发送一个markdown文件，但文件并不存在或文件名过长，将直接发送内容."
                )
        data = {"markdown": {"content": content}}
        return self._send(msg_type="markdown", data=data)

    def send_image(self, image_path=""):
        """
        发送图片类型，限制大小2M，支持JPG，PNG格式
        :param image_path: 图片文件路径
        :return: result
        """
        if not is_image(image_path):
            logger.error(self.errmsgs["image_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["image_error"]}

        img_content = Path(image_path).open("rb").read()
        img_base64 = base64.b64encode(img_content).decode()
        img_md5 = md5(img_content).hexdigest()
        data = {
            "image": {
                "base64": img_base64,
                "md5": img_md5,
            }
        }
        return self._send(msg_type="image", data=data)

    def send_voice(self, voice_path: str):
        """
        发送语音
        :param voice_path:
        :return:
        """
        if not is_voice(voice_path):
            logger.error(self.errmsgs["voice_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["voice_error"]}

        media_res = self._get_media_id(media_type="voice", p_media=Path(voice_path))
        data = {
            "voice": {
                "media_id": media_res.get("media_id", ""),
            }
        }

        return self._send(msg_type="voice", data=data)

    def send_news(self, title="", desp="", url="", picurl=""):
        """
        发送图文消息
        :param title: 图文标题，不超过128个字节，超过会自动截断
        :param desp: 图文描述，可选，不超过512个字节，超过会自动截断
        :param url: 跳转链接
        :param picurl: 图片url，支持JPG、PNG格式，较好的效果为大图 1068*455，小图150*150。
        :return:
        """
        if not (title and url):
            logger.error(self.errmsgs["news_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["news_error"]}
        data = {
            "news": {
                "articles": [
                    {"title": title, "description": desp, "url": url, "picurl": picurl}
                ]
            }
        }
        return self._send(msg_type="news", data=data)

    def send_file(self, file_path: str):
        """
        发送文件
        :param file_path:
        :return:
        """
        if not is_file(file_path):
            logger.error(self.errmsgs["file_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["file_error"]}

        media_res = self._get_media_id(media_type="file", p_media=Path(file_path))
        data = {
            "file": {
                "media_id": media_res.get("media_id", ""),
            }
        }

        return self._send(msg_type="file", data=data)

    def send_template_card(self, template_card, *args, **kwargs):
        data = {"template_card": template_card}

        return self._send(msg_type="template_card", data=data)


class AppMsgSender(MsgSender):
    """
    应用消息推送器，支持文本、图片、语音、视频、文件、文本卡片、图文、markdown消息推送
    """

    def __init__(self, keys_filepath="pten_keys.ini", keys: Keys = None, **kwargs):
        """
        :param corpid: 企业id
        :param corpsecret: 应用密钥
        :param agentid: 应用id
        """
        super().__init__(keys_filepath, keys=keys, **kwargs)
        self.api = CorpApi(keys_filepath, keys=keys)
        self.agentid = self.keys.get_app_agentid()

    def __list2str(self, datas=[]):
        """
        将传入的list数据转换成 | 划分的字符串
        e.g. ['user1', 'user2'] -> 'user1|user2'
        :param datas:
        :return:
        """
        return "".join([item + "|" for item in datas])[:-1]

    def _get_media_id(self, media_type: str, p_media: Path):
        data = {"file": p_media.open("rb")}
        response = self.api.http_call(
            CORP_API_TYPE["MEDIA_UPLOAD"],
            {"type": media_type, "files": data},
        )

        return response

    def _send(
        self,
        msg_type: str = "",
        data: dict = {},
        media_path: Optional[str] = "",
        **kwargs,
    ):
        """
        新的统一内部发送接口，供不同消息推送接口调用
        :param msg_type:
        :param data:
        :param media_path: 只有需要media id的时候才传入
        :param kwargs:
        :return:
        """
        if not (kwargs.get("touser") or kwargs.get("toparty") or kwargs.get("totag")):
            # 三者均为空，默认发送全体成员
            kwargs["touser"] = ["@all"]

        data["chatid"] = kwargs.get("chatid", "")
        if not data["chatid"]:
            # 不是发送到群聊的消息
            data.update(
                {
                    "touser": self.__list2str(kwargs.get("touser", [])),
                    "toparty": self.__list2str(kwargs.get("toparty", [])),
                    "totag": self.__list2str(kwargs.get("totag", [])),
                    "agentid": self.agentid,
                    "enable_id_trans": kwargs.get("enable_id_trans"),
                    "enable_duplicate_check": kwargs.get("enable_duplicate_check"),
                    "duplicate_check_interval": kwargs.get("duplicate_check_interval"),
                }
            )

        data.update(
            {
                "msgtype": msg_type,
                "safe": kwargs.get("safe", 0),
            }
        )

        media_types = {"image", "voice", "video", "file"}
        if msg_type in media_types:
            media_res = self._get_media_id(msg_type, Path(media_path))
            data[msg_type] = {"media_id": media_res.get("media_id", "")}
        elif msg_type == "mpnews":
            media_res = self._get_media_id(media_type="image", p_media=Path(media_path))
            thumb_media_id = media_res.get("media_id", None)
            data[msg_type]["articles"][0]["thumb_media_id"] = thumb_media_id

        if data["chatid"]:
            return self.api.http_call(CORP_API_TYPE["APP_CHAT_SEND"], data)

        return self.api.http_call(CORP_API_TYPE["MESSAGE_SEND"], data)

    def send_text(self, content: str, **kwargs):
        """
        发送text消息
        :param content: 消息内容，最长不超过2048个字节，超过将
        :param safe: 是否是保密消息，False表示可对外分享，True表示不能分享且内容显示水印，默认为False，下面方法同，不再重复解释
        :param kwargs: touser, toparty, totag
        :return: send result
        """
        if not content:
            logger.error(self.errmsgs["text_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["text_error"]}
        data = {
            "text": {"content": content},
        }
        return self._send(msg_type="text", data=data, **kwargs)

    def send_markdown(self, content: str, **kwargs):
        """
        发送markdown消息
        :param content: markdown文本数据或markdown文件路径
        :return:
        """
        if not content:
            logger.error(self.errmsgs["markdown_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["markdown_error"]}
        if content.endswith(".md"):
            try:
                content = Path(content).read_text()
            except (OSError, FileNotFoundError):
                logger.warning(
                    "你可能正在尝试发送一个markdown文件，但文件并不存在或文件名过长，将直接发送内容."
                )
        data = {
            "markdown": {
                "content": content,
            },
        }
        return self._send(msg_type="markdown", data=data, **kwargs)

    def send_image(self, image_path: str, **kwargs):
        """
        发送图片，支持jpg、png、bmp
        :param image_path: 图片存储路径
        :return:
        """
        if not is_image(image_path):
            logger.error(self.errmsgs["image_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["image_error"]}
        return self._send(msg_type="image", media_path=image_path, **kwargs)

    def send_voice(self, voice_path: str, **kwargs):
        """
        发送语音，2MB，播放长度不超过60s，仅支持AMR格式
        :param voice_path:
        :return:
        """
        if not is_voice(voice_path):
            logger.error(self.errmsgs["voice_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["voice_error"]}
        return self._send(msg_type="voice", media_path=voice_path, **kwargs)

    def send_video(self, video_path: str, **kwargs):
        """
        发送视频
        :param video_path:
        :return:
        """
        if not is_video(video_path):
            logger.error(self.errmsgs["video_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["video_error"]}
        return self._send(msg_type="video", media_path=video_path, **kwargs)

    def send_news(
        self, title: str, desp: Optional[str], url: str, picurl: Optional[str], **kwargs
    ):
        """
        发送图文消息
        :param title: 图文标题，不超过128个字节，超过会自动截断
        :param desp: 图文描述，可选，不超过512个字节，超过会自动截断
        :param url: 跳转链接
        :param picurl: 图片url，支持JPG、PNG格式，较好的效果为大图 1068*455，小图150*150。
        :return:
        """
        if not (title and url):
            logger.error(self.errmsgs["news_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["news_error"]}
        data = {
            "news": {
                "articles": [
                    {"title": title, "description": desp, "url": url, "picurl": picurl}
                ]
            },
        }
        return self._send(msg_type="news", data=data, **kwargs)

    def send_file(self, file_path: str, **kwargs):
        """
        发送文件
        :param file_path:
        :return:
        """
        if not is_file(file_path):
            logger.error(self.errmsgs["file_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["file_error"]}
        return self._send(msg_type="file", media_path=file_path, **kwargs)

    def send_mpnews(
        self,
        title: str,
        image_path: str,
        content: str,
        author: Optional[str],
        content_source_url: Optional[str],
        digest: Optional[str],
        **kwargs,
    ):
        """
        发送mpnews消息
        :param title: 图文标题
        :param image_path: 缩略图所在路径
        :param content: 图文消息内容
        :param author: 作者信息
        :param content_source_url: 点击跳转链接
        :param digest: 图文消息描述
        :param kwargs:
        :return:
        """
        if not (title and image_path and content):
            logger.error(self.errmsgs["mpnews_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["mpnews_error"]}
        data = {
            "mpnews": {
                "articles": [
                    {
                        "title": title,
                        "author": author,
                        "content_source_url": content_source_url,
                        "content": content,
                        "digest": digest,
                    }
                ]
            }
        }
        return self._send(msg_type="mpnews", data=data, media_path=image_path, **kwargs)

    def send_card(self, title: str, description: str, url: str, **kwargs):
        """
        发送卡片消息
        :param title: 标题，不超过128个字节，超过会自动截断
        :param desp: 描述，不超过512个字节，超过会自动截断
        :param url: 点击后跳转的链接
        :param btntxt: 按钮文字。 默认为“详情”， 不超过4个文字，超过自动截断
        :return:
        """
        if not (title and description and url):
            logger.error(self.errmsgs["card_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["card_error"]}
        btntxt = kwargs.get("btntxt", None)
        data = {
            "textcard": {
                "title": title,
                "description": description,
                "url": url,
                "btntxt": btntxt,
            },
        }
        return self._send(msg_type="textcard", data=data, **kwargs)

    def send_miniprogram_notice(self, *args, **kwargs):
        raise NotImplementedError

    def send_template_card(self, template_card, *args, **kwargs):
        data = {"template_card": template_card}

        return self._send(msg_type="template_card", data=data, **kwargs)

    def create_chat(
        self,
        name: Optional[str] = "",
        owner: Optional[str] = "",
        userlist: list = [],
        chatid: Optional[str] = "",
        show_chat: Optional[bool] = False,
    ):
        """
        创建应用群聊
        :param users: 用户id列表，至少2人
        :param name: 群聊名称
        :param owner: 群主id，不指定会随机
        :param chatid:
        :param show_chat: 群聊创建成功后是否发送一条消息让群聊在列表中显示出来（默认发送）
        :return:
        """
        if len(userlist) < 2:
            logger.error(self.errmsgs["create_chat_error"])
            return {"errcode": 404, "errmsg": self.errmsgs["create_chat_error"]}

        data = {
            "name": name,
            "owner": owner,
            "userlist": userlist,
            "chatid": chatid,
        }

        res = self.api.http_call(CORP_API_TYPE["APP_CHAT_CREATE"], data)

        if show_chat and res["errcode"] == 0:
            logger.info("group chat is created")
            content = f"group chat is created. chatid: {res['chatid']}"
            self.send_text(content=content, chatid=res["chatid"])
            return res
        else:
            return res



import logging
from logging.handlers import RotatingFileHandler

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class LogColors:
    DEBUG = "\033[96m"
    INFO = "\033[92m"
    WARNING = "\033[93m"
    ERROR = "\033[91m"
    CRITICAL = "\033[41m\033[97m"
    RESET = "\033[0m"


class ColoredFormatter(logging.Formatter):
    def format(self, record):
        level_color = {
            logging.DEBUG: LogColors.DEBUG,
            logging.INFO: LogColors.INFO,
            logging.WARNING: LogColors.WARNING,
            logging.ERROR: LogColors.ERROR,
            logging.CRITICAL: LogColors.CRITICAL,
        }.get(record.levelno, LogColors.RESET)

        message = super().format(record)
        return f"{level_color}{message}{LogColors.RESET}"


formatter = ColoredFormatter(
    "%(asctime)s | %(levelname)s | %(filename)s:%(lineno)d:%(funcName)s | %(message)s"
)
console_handler = logging.StreamHandler()
console_handler.setFormatter(formatter)

formatter = logging.Formatter(
    "%(asctime)s | %(levelname)s | %(filename)s:%(lineno)d:%(funcName)s | %(message)s"
)
maxBytes = 30 * 1024 * 1024  # 30MB
file_handler = RotatingFileHandler(__name__ + ".log", maxBytes=maxBytes, backupCount=3)
file_handler.setFormatter(formatter)

logger.addHandler(file_handler)
logger.addHandler(console_handler)



"""
pten.keys
~~~~~~~~~~~~

This module implements the keys class for getting keys from local file.

"""

from . import logger
import configparser
from configparser import ConfigParser
from datetime import datetime
import json
from pathlib import Path


class Keys:
    """Keys class for getting keys from local file

    :param keys_filepath: The path of the keys file. Default is "pten_keys.ini"
    """

    def __init__(self, keys_filepath="pten_keys.ini", *args, **kwargs):
        self.key_cfg = ConfigParser()
        self.keys_filepath = Path(keys_filepath)
        self.TOKEN_PATH = Path("pten_token.json")
        self.bot_weebhook_key = None
        self.access_token = None
        self.access_token_expire_time = float("-inf")
        self.corp_jsapi_ticket = None
        self.corp_jsapi_ticket_expire_time = float("-inf")
        self.app_jsapi_ticket = None
        self.app_jsapi_ticket_expire_time = float("-inf")

        logger.info(f"keys_filepath : {self.keys_filepath}")

        if not self.keys_filepath.is_file():
            logger.error(f"Can not find file {self.keys_filepath}")

    def _get_local_keys(self, section: str, options=[]):
        """Get keys from local file
        :param section: Section name of the keys
        :param options: The keys you want to get
        :return: generator of the keys
        """
        if self.keys_filepath.is_file():
            self.key_cfg.clear()
            self.key_cfg.read(self.keys_filepath)
            try:
                for option in options:
                    yield self.key_cfg.get(section, option)
            except (configparser.NoSectionError, configparser.NoOptionError):
                raise configparser.Error("KeyConfigError")
        else:
            raise FileNotFoundError(f"Can not find file {self.keys_filepath}")

    def get_key(self, section: str, option: str):
        """Get keys from local file
        :param section: Section name of the keys
        :param option: The key you want to get
        :return: key value
        """
        return next(self._get_local_keys(section, [option]))

    def get_keys(self, section: str, options=[]):
        """Get keys from local file
        :param section: Section name of the keys
        :param options: The keys you want to get
        :return: dict of the keys
        """
        res = {}
        for k, v in zip(options, self._get_local_keys(section, options)):
            res.update({k: v})
        return res

    def get_debug_mode(self):
        debug_mode_str = "false"
        try:
            debug_mode_str = self.get_key("globals", "debug_mode")
        except (configparser.Error, FileNotFoundError):
            debug_mode_str = "false"

        debug_mode = debug_mode_str.lower() in ("true", "yes", "on", "1")
        return debug_mode

    def get_proxies(self):
        proxies = None
        try:
            proxies = self.get_keys(section="proxies", options=["http", "https"])
        except (configparser.Error, FileNotFoundError):
            pass

        return proxies

    def get_bot_weebhook_key(self):
        if self.bot_weebhook_key is None:
            key = next(self._get_local_keys(section="ww", options=["webhook_key"]))
            self.bot_weebhook_key = key

        return self.bot_weebhook_key

    def get_app_agentid(self):
        return next(self._get_local_keys(section="ww", options=["app_agentid"]))

    def get_contact_sync_secret(self):
        try:
            s = self.get_key("ww", "contact_sync_secret")
        except StopIteration:
            logger.warning("Can not find contact_sync_secret in keys ini file")
            s = None
        return s

    @staticmethod
    def load_from_file(file_path: Path, key):
        if not file_path.is_file():
            raise FileNotFoundError(f"Can not find file {file_path}.")

        dict = json.loads(file_path.read_text())
        if key not in dict:
            raise KeyError(f"Can not find token of {key}.")

        return dict[key]

    @staticmethod
    def save_to_file(file_path: Path, key, info):
        token_dict = {}
        if file_path.is_file():
            token_dict = json.loads(file_path.read_text())

        token_dict.update({key: info})
        file_path.write_text(json.dumps(token_dict))

    def get_access_token(self, token_key):
        if self.access_token_expire_time > datetime.now().timestamp():
            return self.access_token

        token_info = Keys.load_from_file(self.TOKEN_PATH, token_key)
        self.access_token_expire_time = token_info.get("expire_time", float("-inf"))
        if self.access_token_expire_time < datetime.now().timestamp():
            logger.warning(f"Token of {token_key} is expired.")
            raise Exception("Token expired")

        self.access_token = token_info["access_token"]
        return self.access_token

    def save_access_token(self, token_key, access_token):
        self.access_token = access_token
        self.access_token_expire_time = datetime.now().timestamp() + 7200
        token_info = {
            "access_token": access_token,
            "expire_time": self.access_token_expire_time,
        }
        Keys.save_to_file(self.TOKEN_PATH, token_key, token_info)

    def get_corp_jsapi_ticket(self, token_key):
        now = datetime.now().timestamp()
        if self.corp_jsapi_ticket_expire_time > now:
            return self.corp_jsapi_ticket

        ticket_key = token_key + "_corp_jsapi_ticket"
        ticket_info = Keys.load_from_file(self.TOKEN_PATH, ticket_key)
        expire_time = ticket_info.get("expire_time", float("-inf"))
        self.corp_jsapi_ticket_expire_time = expire_time
        if expire_time < now:
            logger.warning(f"Ticket of {ticket_key} is expired.")
            raise Exception("Ticket expired")

        self.corp_jsapi_ticket = ticket_info["corp_jsapi_ticket"]
        return self.corp_jsapi_ticket

    def save_corp_jsapi_ticket(self, token_key, corp_jsapi_ticket):
        ticket_key = token_key + "_corp_jsapi_ticket"
        self.corp_jsapi_ticket = corp_jsapi_ticket
        self.corp_jsapi_ticket_expire_time = datetime.now().timestamp() + 7200
        ticket_info = {
            "corp_jsapi_ticket": corp_jsapi_ticket,
            "expire_time": self.corp_jsapi_ticket_expire_time,
        }
        Keys.save_to_file(self.TOKEN_PATH, ticket_key, ticket_info)

    def get_app_jsapi_ticket(self, token_key):
        now = datetime.now().timestamp()
        if self.app_jsapi_ticket_expire_time > now:
            return self.app_jsapi_ticket

        ticket_key = token_key + "_app_jsapi_ticket"
        ticket_info = Keys.load_from_file(self.TOKEN_PATH, ticket_key)
        expire_time = ticket_info.get("expire_time", float("-inf"))
        self.app_jsapi_ticket_expire_time = expire_time
        if expire_time < now:
            logger.warning(f"Ticket of {ticket_key} is expired.")
            raise Exception("Ticket expired")

        self.app_jsapi_ticket = ticket_info["app_jsapi_ticket"]
        return self.app_jsapi_ticket

    def save_app_jsapi_ticket(self, token_key, app_jsapi_ticket):
        ticket_key = token_key + "_app_jsapi_ticket"
        self.app_jsapi_ticket = app_jsapi_ticket
        self.app_jsapi_ticket_expire_time = datetime.now().timestamp() + 7200
        ticket_info = {
            "app_jsapi_ticket": app_jsapi_ticket,
            "expire_time": self.app_jsapi_ticket_expire_time,
        }
        Keys.save_to_file(self.TOKEN_PATH, ticket_key, ticket_info)



"""
pten.wwapi
~~~~~~~~~~~~

This module implements the WeWork API. It is from https://github.com/sbzhu/weworkapi_python

"""

from . import logger
from .keys import Keys
import hashlib
import json
import requests
from urllib.parse import urlencode


class ApiException(Exception):
    def __init__(self, errCode, errMsg):
        self.errCode = errCode
        self.errMsg = errMsg


class AbstractApi(object):
    def __init__(self, keys_filepath="pten_keys.ini", keys: Keys = None):
        self.keys = keys if keys else Keys(keys_filepath)
        self.DEBUG_MODE = self.keys.get_debug_mode()
        self.proxies = self.keys.get_proxies()

    def get_access_token(self):
        raise NotImplementedError

    def refresh_access_token(self):
        raise NotImplementedError

    def get_suite_access_token(self):
        raise NotImplementedError

    def refresh_suite_access_token(self):
        raise NotImplementedError

    def get_provider_access_token(self):
        raise NotImplementedError

    def refresh_provider_access_token(self):
        raise NotImplementedError

    def get_bot_webhook_key(self):
        raise NotImplementedError

    def http_call(self, urlType, args=None):
        shortUrl = urlType[0]
        method = urlType[1]
        response = {}
        for retryCnt in range(0, 3):
            if "POST" == method:
                url = self.__make_url(shortUrl)
                response = self.__http_post(url, args)
            elif "GET" == method:
                url = self.__make_url(shortUrl)
                url = self.__append_args(url, args)
                response = self.__http_get(url)
            elif "POST_FILE" == method:
                url = self.__make_url(shortUrl)
                response = self.__post_file(url, args)
            else:
                raise ApiException(-1, "unknown method type")

            # check if token expired
            if self.__token_expired(response.get("errcode")):
                self.__refresh_token(shortUrl)
                retryCnt += 1
                continue
            else:
                break

        return self.__check_response(response)

    @staticmethod
    def __append_args(url, args):
        if args is None:
            return url

        for key, value in args.items():
            if "?" in url:
                url += "&" + key + "=" + value
            else:
                url += "?" + key + "=" + value
        return url

    @staticmethod
    def __make_url(shortUrl):
        base = "https://qyapi.weixin.qq.com"
        if shortUrl[0] == "/":
            return base + shortUrl
        else:
            return base + "/" + shortUrl

    def __appendToken(self, url):
        if "SUITE_ACCESS_TOKEN" in url:
            return url.replace("SUITE_ACCESS_TOKEN", self.get_suite_access_token())
        elif "PROVIDER_ACCESS_TOKEN" in url:
            return url.replace(
                "PROVIDER_ACCESS_TOKEN", self.get_provider_access_token()
            )
        elif "ACCESS_TOKEN" in url:
            return url.replace("ACCESS_TOKEN", self.get_access_token())
        elif "WEBHOOK_KEY" in url:
            return url.replace("WEBHOOK_KEY", self.get_bot_webhook_key())
        else:
            return url

    def __http_post(self, url, args):
        realUrl = self.__appendToken(url)

        if self.DEBUG_MODE is True:
            realUrl += "&debug=1"
            query_string = urlencode(args)
            full_url = f"{realUrl}?{query_string}"
            logger.debug(full_url)

        return requests.post(
            realUrl,
            data=json.dumps(args, ensure_ascii=False).encode("utf-8"),
            proxies=self.proxies,
        ).json()

    def __http_get(self, url):
        realUrl = self.__appendToken(url)

        if self.DEBUG_MODE is True:
            realUrl += "&debug=1"
            logger.debug(realUrl)

        return requests.get(realUrl, proxies=self.proxies).json()

    def __post_file(self, url, args):
        realUrl = self.__appendToken(url)

        type = args.get("type", None)
        files = args.get("files", None)
        if type is None or files is None:
            raise ApiException(-1, "type is None or file is None")

        realUrl = self.__append_args(realUrl, {"type": type})

        return requests.post(realUrl, files=files, proxies=self.proxies).json()

    @staticmethod
    def __check_response(response):
        errCode = response.get("errcode")
        errMsg = response.get("errmsg")

        if errCode == 0:
            return response
        else:
            raise ApiException(errCode, errMsg)

    @staticmethod
    def __token_expired(errCode):
        if errCode == 40014 or errCode == 42001 or errCode == 42007 or errCode == 42009:
            return True
        else:
            return False

    def __refresh_token(self, url):
        if "SUITE_ACCESS_TOKEN" in url:
            self.refresh_suite_access_token()
        elif "PROVIDER_ACCESS_TOKEN" in url:
            self.refresh_provider_access_token()
        elif "ACCESS_TOKEN" in url:
            self.refresh_access_token()


BOT_API_TYPE = {
    # bot using webhook key. No access_token
    "WEBHOOK_SEND": ["/cgi-bin/webhook/send?key=WEBHOOK_KEY", "POST"],
    "WEBHOOK_MEDIA_UPLOAD": [
        "/cgi-bin/webhook/upload_media?key=WEBHOOK_KEY",
        "POST_FILE",
    ],
}


class BotApi(AbstractApi):
    def __init__(
        self, keys_filepath="pten_keys.ini", webhook_key=None, keys: Keys = None
    ):
        super().__init__(keys_filepath, keys=keys)
        if webhook_key is not None:
            self.webhook_key = webhook_key
        else:
            self.webhook_key = self.keys.get_bot_weebhook_key()

    def get_bot_webhook_key(self):
        return self.webhook_key


CORP_API_TYPE = {
    "GET_ACCESS_TOKEN": ["/cgi-bin/gettoken", "GET"],
    # user
    "USER_CREATE": ["/cgi-bin/user/create?access_token=ACCESS_TOKEN", "POST"],
    "USER_GET": ["/cgi-bin/user/get?access_token=ACCESS_TOKEN", "GET"],
    "USER_UPDATE": ["/cgi-bin/user/update?access_token=ACCESS_TOKEN", "POST"],
    "USER_DELETE": ["/cgi-bin/user/delete?access_token=ACCESS_TOKEN", "GET"],
    "USER_BATCH_DELETE": [
        "/cgi-bin/user/batchdelete?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "USER_SIMPLE_LIST": ["/cgi-bin/user/simplelist?access_token=ACCESS_TOKEN", "GET"],
    "USER_LIST": ["/cgi-bin/user/list?access_token=ACCESS_TOKEN", "GET"],
    "USERID_TO_OPENID": [
        "/cgi-bin/user/convert_to_openid?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "OPENID_TO_USERID": [
        "/cgi-bin/user/convert_to_userid?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "GET_USERID_BY_MOBILE": [
        "cgi-bin/user/getuserid?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "GET_USERID_BY_EMAIL": [
        "cgi-bin/user/get_userid_by_email?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "USER_ID_LIST": [
        "cgi-bin/user/list_id?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "USER_AUTH_SUCCESS": ["/cgi-bin/user/authsucc?access_token=ACCESS_TOKEN", "GET"],
    "GET_USER_INFO_BY_CODE": [
        "/cgi-bin/user/getuserinfo?access_token=ACCESS_TOKEN",
        "GET",
    ],
    "GET_USER_DETAIL": [
        "/cgi-bin/user/getuserdetail?access_token=ACCESS_TOKEN",
        "POST",
    ],
    # department
    "DEPARTMENT_CREATE": [
        "/cgi-bin/department/create?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "DEPARTMENT_UPDATE": [
        "/cgi-bin/department/update?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "DEPARTMENT_DELETE": [
        "/cgi-bin/department/delete?access_token=ACCESS_TOKEN",
        "GET",
    ],
    "DEPARTMENT_LIST": ["/cgi-bin/department/list?access_token=ACCESS_TOKEN", "GET"],
    "DEPARTMENT_SIMPLE_LIST": [
        "/cgi-bin/department/simplelist?access_token=ACCESS_TOKEN",
        "GET",
    ],
    "DEPARTMENT_GET": ["/cgi-bin/department/get?access_token=ACCESS_TOKEN", "GET"],
    # tag
    "TAG_CREATE": ["/cgi-bin/tag/create?access_token=ACCESS_TOKEN", "POST"],
    "TAG_UPDATE": ["/cgi-bin/tag/update?access_token=ACCESS_TOKEN", "POST"],
    "TAG_DELETE": ["/cgi-bin/tag/delete?access_token=ACCESS_TOKEN", "GET"],
    "TAG_GET_USER": ["/cgi-bin/tag/get?access_token=ACCESS_TOKEN", "GET"],
    "TAG_ADD_USER": ["/cgi-bin/tag/addtagusers?access_token=ACCESS_TOKEN", "POST"],
    "TAG_DELETE_USER": ["/cgi-bin/tag/deltagusers?access_token=ACCESS_TOKEN", "POST"],
    "TAG_GET_LIST": ["/cgi-bin/tag/list?access_token=ACCESS_TOKEN", "GET"],
    # batch
    "BATCH_INVITE": ["/cgi-bin/batch/invite?access_token=ACCESS_TOKEN", "POST"],
    "BATCH_SYNC_USERS": ["/cgi-bin/batch/syncuser?access_token=ACCESS_TOKEN", "POST"],
    "BATCH_REPLACE_USERS": [
        "/cgi-bin/batch/replaceuser?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "BATCH_REPLACE_PARTY": [
        "/cgi-bin/batch/replaceparty?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "BATCH_JOB_GET_RESULT": [
        "/cgi-bin/batch/getresult?access_token=ACCESS_TOKEN",
        "GET",
    ],
    # export
    "EXPORT_SIMPLE_USER": [
        "/cgi-bin/export/simple_user?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "EXPORT_USER": [
        "/cgi-bin/export/user?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "EXPORT_DEPARTMENT": [
        "/cgi-bin/export/department?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "EXPORT_TAG_USER": [
        "/cgi-bin/export/taguser?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "EXPORT_GET_RESULT": [
        "/cgi-bin/export/get_result?access_token=ACCESS_TOKEN",
        "POST",
    ],
    # auth
    "AUTH_GET_USER_INFO": [
        "/cgi-bin/auth/getuserinfo?access_token=ACCESS_TOKEN",
        "GET",
    ],
    "AUTH_GET_USER_DETAIL": [
        "/cgi-bin/auth/getuserdetail?access_token=ACCESS_TOKEN",
        "POST",
    ],
    # agent
    "AGENT_GET": ["/cgi-bin/agent/get?access_token=ACCESS_TOKEN", "GET"],
    "AGENT_SET": ["/cgi-bin/agent/set?access_token=ACCESS_TOKEN", "POST"],
    "AGENT_GET_LIST": ["/cgi-bin/agent/list?access_token=ACCESS_TOKEN", "GET"],
    # menu
    "MENU_CREATE": ["/cgi-bin/menu/create?access_token=ACCESS_TOKEN", "POST"],
    "MENU_GET": ["/cgi-bin/menu/get?access_token=ACCESS_TOKEN", "GET"],
    "MENU_DELETE": ["/cgi-bin/menu/delete?access_token=ACCESS_TOKEN", "GET"],
    # message
    "MESSAGE_SEND": ["/cgi-bin/message/send?access_token=ACCESS_TOKEN", "POST"],
    "MESSAGE_REVOKE": ["/cgi-bin/message/revoke?access_token=ACCESS_TOKEN", "POST"],
    # media
    "MEDIA_GET": ["/cgi-bin/media/get?access_token=ACCESS_TOKEN", "GET"],
    "MEDIA_UPLOAD": ["/cgi-bin/media/upload?access_token=ACCESS_TOKEN", "POST_FILE"],
    # ticket
    "GET_TICKET": ["/cgi-bin/ticket/get?access_token=ACCESS_TOKEN", "GET"],
    "GET_JSAPI_TICKET": ["/cgi-bin/get_jsapi_ticket?access_token=ACCESS_TOKEN", "GET"],
    # checkin
    "GET_CHECKIN_OPTION": [
        "/cgi-bin/checkin/getcheckinoption?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "GET_CHECKIN_DATA": [
        "/cgi-bin/checkin/getcheckindata?access_token=ACCESS_TOKEN",
        "POST",
    ],
    # corp
    "GET_JOIN_QRCODE": [
        "/cgi-bin/corp/get_join_qrcode?access_token=ACCESS_TOKEN",
        "GET",
    ],
    "GET_APPROVAL_DATA": [
        "/cgi-bin/corp/getapprovaldata?access_token=ACCESS_TOKEN",
        "POST",
    ],
    # card
    "GET_INVOICE_INFO": [
        "/cgi-bin/card/invoice/reimburse/getinvoiceinfo?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "UPDATE_INVOICE_STATUS": [
        "/cgi-bin/card/invoice/reimburse/updateinvoicestatus?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "BATCH_UPDATE_INVOICE_STATUS": [
        "/cgi-bin/card/invoice/reimburse/updatestatusbatch?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "BATCH_GET_INVOICE_INFO": [
        "/cgi-bin/card/invoice/reimburse/getinvoiceinfobatch?access_token=ACCESS_TOKEN",
        "POST",
    ],
    # appchat
    "APP_CHAT_CREATE": ["/cgi-bin/appchat/create?access_token=ACCESS_TOKEN", "POST"],
    "APP_CHAT_GET": ["/cgi-bin/appchat/get?access_token=ACCESS_TOKEN", "GET"],
    "APP_CHAT_UPDATE": ["/cgi-bin/appchat/update?access_token=ACCESS_TOKEN", "POST"],
    "APP_CHAT_SEND": ["/cgi-bin/appchat/send?access_token=ACCESS_TOKEN", "POST"],
    # miniprogram
    "MINIPROGRAM_CODE_TO_SESSION_KEY": [
        "/cgi-bin/miniprogram/jscode2session?access_token=ACCESS_TOKEN",
        "GET",
    ],
    # wedoc
    "WEDOC_CREATE": ["/cgi-bin/wedoc/create_doc?access_token=ACCESS_TOKEN", "POST"],
    "WEDOC_RENAME": ["/cgi-bin/wedoc/rename_doc?access_token=ACCESS_TOKEN", "POST"],
    "WEDOC_DELETE": ["/cgi-bin/wedoc/del_doc?access_token=ACCESS_TOKEN", "POST"],
    "WEDOC_GET_BASIC_INFO": [
        "/cgi-bin/wedoc/get_doc_base_info?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SHARE": ["/cgi-bin/wedoc/doc_share?access_token=ACCESS_TOKEN", "POST"],
    "WEDOC_BATCH_UPDATE": [
        "/cgi-bin/wedoc/document/batch_update?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_GET_DOC": ["/cgi-bin/wedoc/document/get?access_token=ACCESS_TOKEN", "POST"],
    "WEDOC_SPREADSHEET_BATCH_UPDATE": [
        "/cgi-bin/wedoc/spreadsheet/batch_update?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SPREADSHEET_GET_RANGE_DATA": [
        "/cgi-bin/wedoc/spreadsheet/get_sheet_range_data?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SPREADSHEET_GET_PROPERTITIES": [
        "/cgi-bin/wedoc/spreadsheet/get_sheet_properties?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_ADD_SHEET": [
        "/cgi-bin/wedoc/smartsheet/add_sheet?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_DELETE_SHEET": [
        "/cgi-bin/wedoc/smartsheet/delete_sheet?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_UPDATE_SHEET": [
        "/cgi-bin/wedoc/smartsheet/update_sheet?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_ADD_VIEW": [
        "/cgi-bin/wedoc/smartsheet/add_view?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_DELETE_VIEW": [
        "/cgi-bin/wedoc/smartsheet/delete_views?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_UPDATE_VIEW": [
        "/cgi-bin/wedoc/smartsheet/update_view?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_ADD_FIELDS": [
        "/cgi-bin/wedoc/smartsheet/add_fields?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_DELETE_FIELDS": [
        "/cgi-bin/wedoc/smartsheet/delete_fields?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_UPDATE_FIELDS": [
        "/cgi-bin/wedoc/smartsheet/update_fields?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_ADD_RECORDS": [
        "/cgi-bin/wedoc/smartsheet/add_records?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_DELETE_RECORDS": [
        "/cgi-bin/wedoc/smartsheet/delete_records?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_UPDATE_RECORDS": [
        "/cgi-bin/wedoc/smartsheet/update_records?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_GET_SHEET": [
        "/cgi-bin/wedoc/smartsheet/get_sheet?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_GET_VIEWS": [
        "/cgi-bin/wedoc/smartsheet/get_views?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_GET_FIELDS": [
        "/cgi-bin/wedoc/smartsheet/get_fields?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_GET_RECORDS": [
        "/cgi-bin/wedoc/smartsheet/get_records?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_SMARTSHEET_GET_SHEET_PRIV": [
        "/cgi-bin/wedoc/smartsheet/content_priv/get_sheet_priv?access_token=ACCESS_TOKEN",
        "POST",
    ],
    # smartsheet/content_priv 还有好几个接口这里没有添加进来，有需要可以参考下面的链接来添加
    # https://developer.work.weixin.qq.com/document/path/100193
    "WEDOC_GET_DOC_AUTH": [
        "/cgi-bin/wedoc/doc_get_auth?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_MOD_DOC_JOIN_RULE": [
        "/cgi-bin/wedoc/mod_doc_join_rule?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_MOD_DOC_MEMBER": [
        "/cgi-bin/wedoc/mod_doc_member?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_MOD_DOC_SAFETY_SETTING": [
        "/cgi-bin/wedoc/mod_doc_safty_setting?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_CREATE_FORM": [
        "/cgi-bin/wedoc/create_form?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_MODIFY_FORM": [
        "/cgi-bin/wedoc/modify_form?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_GET_FORM_INFO": [
        "/cgi-bin/wedoc/get_form_info?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_GET_FORM_STATISTIC": [
        "/cgi-bin/wedoc/get_form_statistic?access_token=ACCESS_TOKEN",
        "POST",
    ],
    "WEDOC_GET_FORM_ANSWER": [
        "/cgi-bin/wedoc/get_form_answer?access_token=ACCESS_TOKEN",
        "POST",
    ],
}


class CorpApi(AbstractApi):
    def __init__(
        self,
        keys_filepath="pten_keys.ini",
        corpid=None,
        corpsecret=None,
        keys: Keys = None,
    ):
        super().__init__(keys_filepath, keys=keys)
        self.corpid = corpid if corpid else self.keys.get_key("ww", "corpid")
        corpsecret_in_file = self.keys.get_key("ww", "app_secret")
        self.corpsecret = corpsecret if corpsecret else corpsecret_in_file

        self._token_key = hashlib.sha1(
            bytes(self.corpid + self.corpsecret, encoding="utf-8")
        ).hexdigest()

    def get_access_token(self):
        try:
            return self.keys.get_access_token(self._token_key)
        except Exception as e:
            logger.warning(f"{str(e)} refreshing access token...")
            return self.refresh_access_token()

    def refresh_access_token(self):
        response = self.http_call(
            CORP_API_TYPE["GET_ACCESS_TOKEN"],
            {
                "corpid": self.corpid,
                "corpsecret": self.corpsecret,
            },
        )
        access_token = response.get("access_token")
        self.keys.save_access_token(self._token_key, access_token)

        return access_token

    def get_corp_jsapi_ticket(self):
        try:
            return self.keys.get_corp_jsapi_ticket(self._token_key)
        except Exception as e:
            logger.warning(f"{str(e)} refreshing corp jsapi ticket...")
            return self.refresh_corp_jsapi_ticket()

    def refresh_corp_jsapi_ticket(self):
        response = self.http_call(CORP_API_TYPE["GET_JSAPI_TICKET"])
        corp_jsapi_ticket = response.get("ticket")
        self.keys.save_corp_jsapi_ticket(self._token_key, corp_jsapi_ticket)

        return corp_jsapi_ticket

    def get_app_jsapi_ticket(self):
        try:
            return self.keys.get_app_jsapi_ticket(self._token_key)
        except Exception as e:
            logger.warning(f"{str(e)} refreshing app jsapi ticket...")
            return self.refresh_app_jsapi_ticket()

    def refresh_app_jsapi_ticket(self):
        response = self.http_call(CORP_API_TYPE["GET_TICKET"], {"type": "agent_config"})
        app_jsapi_ticket = response.get("ticket")
        self.keys.save_app_jsapi_ticket(self._token_key, app_jsapi_ticket)

        return app_jsapi_ticket


SERVICE_CORP_API_TYPE = {
    "GET_CORP_TOKEN": [
        "/cgi-bin/service/get_corp_token?suite_access_token=SUITE_ACCESS_TOKEN",
        "POST",
    ],
    "GET_SUITE_TOKEN": ["/cgi-bin/service/get_suite_token", "POST"],
    "GET_PRE_AUTH_CODE": [
        "/cgi-bin/service/get_pre_auth_code?suite_access_token=SUITE_ACCESS_TOKEN",
        "GET",
    ],
    "SET_SESSION_INFO": [
        "/cgi-bin/service/set_session_info?suite_access_token=SUITE_ACCESS_TOKEN",
        "POST",
    ],
    "GET_PERMANENT_CODE": [
        "/cgi-bin/service/get_permanent_code?suite_access_token=SUITE_ACCESS_TOKEN",
        "POST",
    ],
    "GET_AUTH_INFO": [
        "/cgi-bin/service/get_auth_info?suite_access_token=SUITE_ACCESS_TOKEN",
        "POST",
    ],
    "GET_ADMIN_LIST": [
        "/cgi-bin/service/get_admin_list?suite_access_token=SUITE_ACCESS_TOKEN",
        "POST",
    ],
    "GET_USER_INFO_BY_3RD": [
        "/cgi-bin/service/getuserinfo3rd?suite_access_token=SUITE_ACCESS_TOKEN",
        "GET",
    ],
    "GET_USER_DETAIL_BY_3RD": [
        "/cgi-bin/service/getuserdetail3rd?suite_access_token=SUITE_ACCESS_TOKEN",
        "POST",
    ],
}


class ServiceCorpApi(CorpApi):
    def __init__(
        self,
        suite_id,
        suite_secret,
        suite_ticket,
        keys_filepath="pten_keys.ini",
        keys: Keys = None,
        auth_corpid=None,
        permanent_code=None,
    ):
        super().__init__(keys_filepath, keys=keys)
        self.suite_id = suite_id
        self.suite_secret = suite_secret
        self.suite_ticket = suite_ticket

        # 调用 CorpAPI 的function， 需要设置这两个参数
        self.auth_corpid = auth_corpid
        self.permanent_code = permanent_code

        self.access_token = None
        self.suite_access_token = None

    ## override CorpApi 的 refreshAccessToken， 使用第三方服务商的方法
    def get_access_token(self):
        if self.access_token is None:
            self.refresh_access_token()
        return self.access_token

    def refresh_access_token(self):
        response = self.http_call(
            SERVICE_CORP_API_TYPE["GET_CORP_TOKEN"],
            {
                "auth_corpid": self.auth_corpid,
                "permanent_code": self.permanent_code,
            },
        )
        self.access_token = response.get("access_token")

    ##
    def get_suite_access_token(self):
        if self.suite_access_token is None:
            self.refresh_suite_access_token()
        return self.suite_access_token

    def refresh_suite_access_token(self):
        response = self.http_call(
            SERVICE_CORP_API_TYPE["GET_SUITE_TOKEN"],
            {
                "suite_id": self.suite_id,
                "suite_secret": self.suite_secret,
                "suite_ticket": self.suite_ticket,
            },
        )
        self.suite_access_token = response.get("suite_access_token")


SERVICE_PROVIDER_API_TYPE = {
    "GET_PROVIDER_TOKEN": ["/cgi-bin/service/get_provider_token", "POST"],
    "GET_LOGIN_INFO": [
        "/cgi-bin/service/get_login_info?access_token=PROVIDER_ACCESS_TOKEN",
        "POST",
    ],
    "GET_REGISTER_CODE": [
        "/cgi-bin/service/get_register_code?provider_access_token=PROVIDER_ACCESS_TOKEN",
        "POST",
    ],
    "GET_REGISTER_INFO": [
        "/cgi-bin/service/get_register_info?provider_access_token=PROVIDER_ACCESS_TOKEN",
        "POST",
    ],
    "SET_AGENT_SCOPE": ["/cgi-bin/agent/set_scope", "POST"],
    "SET_CONTACT_SYNC_SUCCESS": ["/cgi-bin/sync/contact_sync_success", "GET"],
}


class ServiceProviderApi(AbstractApi):
    def __init__(
        self, corpid, provider_secret, keys_filepath="pten_keys.ini", keys: Keys = None
    ):
        super().__init__(keys_filepath, keys=keys)
        self.corpid = corpid
        self.provider_secret = provider_secret
        self.provider_access_token = None

    def get_provider_access_token(self):
        if self.provider_access_token is None:
            self.refresh_provider_access_token()
        return self.provider_access_token

    def refresh_provider_access_token(self):
        response = self.http_call(
            SERVICE_PROVIDER_API_TYPE["GET_PROVIDER_TOKEN"],
            {
                "corpid": self.corpid,
                "provider_secret": self.provider_secret,
            },
        )
        self.provider_access_token = response.get("provider_access_token")



"""
pten.wwdoc
~~~~~~~~~~~~

This module implements the Doc class.

"""

from .keys import Keys
from .wwapi import CorpApi, CORP_API_TYPE


class Doc:
    """
    应用在调用文档接口前，需要先获得文档的使用权限：
    自建应用: 登录企业微信管理端，进入「协作」-「文档」-「API」,配置「可调用接口的应用」
    第三方应用: 第三方服务商创建应用的时候，需要开启“文档接口权限”，企业授权安装第三方应用之后，第三方应用即拥有了调用文档接口的权限
    代开发自建应用: 第三方服务商为企业配置代开发应用时，需要开启「文档接口权限」，企业管理员确认之后，应用即拥有文档权限
    """

    def __init__(self, keys_filepath="pten_keys.ini", keys: Keys = None, **kwargs):
        self.keys = keys if keys else Keys(keys_filepath)
        self.api = CorpApi(keys_filepath, keys=keys)

    def create_doc(self, doc_type, doc_name, **kwargs):
        data = {"doc_type": doc_type, "doc_name": doc_name}
        data.update(kwargs)

        return self.api.http_call(CORP_API_TYPE["WEDOC_CREATE"], data)

    def solve_docid_formid(self, docid, formid):
        if docid and formid:
            raise ValueError("docid and formid cannot be provided at the same time")

        if docid is None and formid is None:
            raise ValueError("docid or formid must be provided")

        data = {}
        if docid is not None:
            data["docid"] = docid
        else:
            data["formid"] = formid

        return data

    def rename_doc(self, new_name, docid=None, formid=None):
        data = self.solve_docid_formid(docid, formid)
        data = {"new_name": new_name}

        return self.api.http_call(CORP_API_TYPE["WEDOC_RENAME"], data)

    def delete_doc(self, docid, formid=None):
        """
        仅可删除应用自己创建的文档和收集表
        """
        data = self.solve_docid_formid(docid, formid)
        return self.api.http_call(CORP_API_TYPE["WEDOC_DELETE"], data)

    def get_doc_base_info(self, docid):
        data = {"docid": docid}
        return self.api.http_call(CORP_API_TYPE["WEDOC_GET_BASIC_INFO"], data)

    def get_doc_share_url(self, docid=None, formid=None):
        data = self.solve_docid_formid(docid, formid)
        return self.api.http_call(CORP_API_TYPE["WEDOC_SHARE"], data)

    def udpate_doc(self, docid, requests, version=None):
        """
        https://developer.work.weixin.qq.com/document/path/97626  里有多种编辑文档的操作
        """
        data = {"docid": docid, "requests": requests, "version": version}
        if version:
            data["version"] = version

        return self.api.http_call(CORP_API_TYPE["WEDOC_BATCH_UPDATE"], data)

    def update_spreadsheet(self, docid, requests):
        """
        https://developer.work.weixin.qq.com/document/path/97628  里有多种编辑文档的操作
        """
        data = {"docid": docid, "requests": requests}
        return self.api.http_call(CORP_API_TYPE["WEDOC_SPREADSHEET_BATCH_UPDATE"], data)

    def get_doc(self, docid):
        """
        用于获取文档数据  https://developer.work.weixin.qq.com/document/path/97659
        """
        return self.api.http_call(CORP_API_TYPE["WEDOC_GET_DOC"], {"docid": docid})

    def get_spreadsheet_properties(self, docid):
        return self.api.http_call(
            CORP_API_TYPE["WEDOC_SPREADSHEET_GET_PROPERTITIES"], {"docid": docid}
        )

    def get_spreadsheet_range_data(self, docid, sheetid, range):
        return self.api.http_call(
            CORP_API_TYPE["WEDOC_SPREADSHEET_GET_RANGE_DATA"],
            {"docid": docid, "sheet_id": sheetid, "range": range},
        )

    def smartsheet_add_sheet(self, docid, properties):
        data = {"docid": docid, "properties": properties}
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_ADD_SHEET"], data)

    def smartsheet_delete_sheet(self, docid, sheet_id):
        data = {"docid": docid, "sheet_id": sheet_id}
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_DELETE_SHEET"], data)

    def smartsheet_update_sheet(self, docid, sheet_id, title):
        """
        用于修改表格中某个子表的标题。
        """
        data = {"docid": docid, "properties": {"sheet_id": sheet_id, "title": title}}
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_UPDATE_SHEET"], data)

    def smartsheet_add_view(self, docid, sheet_id, view_title, view_type, **kwargs):
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
            "view_title": view_title,
            "view_type": view_type,
        }
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_ADD_VIEW"], data)

    def smartsheet_delete_view(self, docid, sheet_id, view_ids):
        data = {"docid": docid, "sheet_id": sheet_id, "view_ids": view_ids}
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_DELETE_VIEW"], data)

    def smartsheet_update_view(
        self, docid, sheet_id, view_id, view_title=None, **kwargs
    ):
        """
        https://developer.work.weixin.qq.com/document/path/100219 可查看property
        """
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
            "view_id": view_id,
        }
        if view_title:
            data["view_title"] = view_title
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_UPDATE_VIEW"], data)

    def smartsheet_add_fields(self, docid, sheet_id, fields):
        """
        https://developer.work.weixin.qq.com/document/path/100220
        """
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
            "fields": fields,
        }
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_ADD_FIELDS"], data)

    def smartsheet_delete_fields(self, docid, sheet_id, field_ids):
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
            "field_ids": field_ids,
        }
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_DELETE_FIELDS"], data)

    def smartsheet_update_fields(self, docid, sheet_id, fields):
        """
        https://developer.work.weixin.qq.com/document/path/100222
        该接口只能更新字段名、字段属性，不能更新字段类型。
        """
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
            "fields": fields,
        }
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_UPDATE_FIELDS"], data)

    def smartsheet_add_records(self, docid, sheet_id, records, key_type=None):
        """
        https://developer.work.weixin.qq.com/document/path/100224
        records里values的 key 为字段标题或字段 ID, 由key_type决定
        """
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
            "key_type": "CELL_VALUE_KEY_TYPE_FIELD_TITLE",
            "records": records,
        }
        if key_type:
            data["key_type"] = key_type
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_ADD_RECORDS"], data)

    def smartsheet_delete_records(self, docid, sheet_id, record_ids):
        return self.api.http_call(
            CORP_API_TYPE["WEDOC_SMARTSHEET_DELETE_RECORDS"],
            {"docid": docid, "sheet_id": sheet_id, "record_ids": record_ids},
        )

    def smartsheet_update_records(self, docid, sheet_id, update_records, key_type=None):
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
            "records": update_records,
        }
        if key_type:
            data["key_type"] = key_type

        return self.api.http_call(
            CORP_API_TYPE["WEDOC_SMARTSHEET_UPDATE_RECORDS"], data
        )

    def smartsheet_get_sheet(self, docid, sheet_id=None, need_all_type_sheet=True):
        data = {"docid": docid}
        if sheet_id:
            data["sheet_id"] = sheet_id
        if need_all_type_sheet:
            data["need_all_type_sheet"] = need_all_type_sheet

        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_GET_SHEET"], data)

    def smartsheet_get_views(self, docid, sheet_id, views_ids=None, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/100228  信息说明
        """
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
        }
        if views_ids:
            data["views_ids"] = views_ids
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_GET_VIEWS"], data)

    def smartsheet_get_fields(self, docid, sheet_id, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/100229
        """
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
        }
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_GET_FIELDS"], data)

    def smartsheet_get_records(self, docid, sheet_id, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/100230
        """
        data = {
            "docid": docid,
            "sheet_id": sheet_id,
            "offset": 0,
            "limit": 1000,
        }
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_SMARTSHEET_GET_RECORDS"], data)

    def smartsheet_get_sheet_priv(self, docid, type, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/100193
        type 权限规则类型, 1: 全员权限; 2: 额外权限
        查询额外权限(type=2)时, 需要添加rule_id_list
        """
        data = {"docid": docid, "type": type}
        data.update(kwargs)
        return self.api.http_call(
            CORP_API_TYPE["WEDOC_SMARTSHEET_GET_SHEET_PRIV"], data
        )

    def get_doc_auth(self, docid):
        return self.api.http_call(CORP_API_TYPE["WEDOC_GET_DOC_AUTH"], {"docid": docid})

    def mod_doc_join_rule(self, docid, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/97792
        """
        data = {"docid": docid}
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_MOD_DOC_JOIN_RULE"], data)

    def mod_doc_member(self, docid, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/97795
        """
        data = {"docid": docid}
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_MOD_DOC_MEMBER"], data)

    def mod_doc_safety_setting(self, docid, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/97797
        """
        data = {"docid": docid}
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_MOD_DOC_SAFETY_SETTING"], data)

    def create_form(self, form_info, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/97472
        """
        data = {"form_info": form_info}
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_CREATE_FORM"], data)

    def modify_form(self, oper, formid, form_info, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/97820
        """
        data = {"oper": oper, "formid": formid, "form_info": form_info}
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_MODIFY_FORM"], data)

    def get_form_info(self, formid):
        data = {"formid": formid}
        return self.api.http_call(CORP_API_TYPE["WEDOC_GET_FORM_INFO"], data)

    def get_form_statistic(self, repeated_id, req_type=1, **kwargs):
        """
        https://developer.work.weixin.qq.com/document/path/97822
        """
        data = {"repeated_id": repeated_id, "req_type": req_type}
        data.update(kwargs)
        return self.api.http_call(CORP_API_TYPE["WEDOC_GET_FORM_STATISTIC"], data)

    def get_form_answer(self, repeated_id, answer_ids):
        """
        https://developer.work.weixin.qq.com/document/path/97823
        """
        data = {"repeated_id": repeated_id, "answer_ids": answer_ids}
        return self.api.http_call(CORP_API_TYPE["WEDOC_GET_FORM_ANSWER"], data)



from setuptools import setup, find_packages

requires = ["apscheduler", "lunardate", "openai", "pycryptodome"]
test_requirements = [
    "pytest>=3",
    "pytest-mock>=3",
]

setup(
    name="pten",
    version="0.2.0",
    description="A tool to use Wework API quickly and easily",
    author="PENGyong",
    author_email="1203029076@qq.com",
    url="https://github.com/bendell02/pten",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">=3.8",
    install_requires=requires,
    tests_require=test_requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords="wework qywx wechat weixin robot app",
)



"""
pten.notice
~~~~~~~~~~~~

This module implements the notice functions.

"""

from . import logger
from .keys import Keys
from apscheduler.schedulers.blocking import BaseScheduler
import datetime
from lunardate import LunarDate
from openai import OpenAI
import requests
import json


class Notice:
    def __init__(self):
        self.report_func = print
        self.scheduler: BaseScheduler = None

    def set_report_func(self, func):
        self.report_func = func

    def set_scheduler(self, scheduler: BaseScheduler):
        self.scheduler = scheduler

    def report_text(self, text):
        self.report_func(text)


class Birthday(Notice):
    def __init__(self):
        super().__init__()

    @staticmethod
    def get_date_str_from_lunar_date(
        lunar_year, lunar_month, lunar_day, hour=8, minute=3, is_leap_month=False
    ):
        lunar_date = LunarDate(lunar_year, lunar_month, lunar_day, is_leap_month)
        solar_date = lunar_date.toSolarDate()
        return f"{solar_date} {hour:02d}:{minute:02d}:00"

    @staticmethod
    def is_leap_month(lunar_year, lunar_month):
        return lunar_month == LunarDate.leapMonthForYear(lunar_year)

    @staticmethod
    def get_now_lunar_date():
        now_solar_date = datetime.datetime.now()
        year = now_solar_date.year
        month = now_solar_date.month
        day = now_solar_date.day
        now_lunar_date = LunarDate.fromSolarDate(year, month, day)
        return now_lunar_date

    @staticmethod
    def generate_birthday_greeting(who, greeting_words=None):
        msg = f"今天是{who}的生日，让我们来祝福{who}吧"
        if greeting_words:
            msg = f"今天是{who}的生日，{greeting_words}"

        return msg

    def _add_lunar_schedule(
        self, msg, lunar_year, lunar_month, lunar_day, hour, minute
    ):
        run_date = self.get_date_str_from_lunar_date(
            lunar_year, lunar_month, lunar_day, hour, minute, False
        )
        args = [msg, lunar_month, lunar_day, hour, minute]
        func = self.report_lunar_birthday
        self.scheduler.add_job(func, "date", run_date=run_date, args=args)

        # solve leap month
        if not self.is_leap_month(lunar_year, lunar_month):
            return
        run_date = self.get_date_str_from_lunar_date(
            lunar_year, lunar_month, lunar_day, hour, minute, True
        )
        self.scheduler.add_job(func, "date", run_date=run_date, args=args)

    def _add_solar_schedule(self, msg, year, month, day, hour, minute):
        solar_date = datetime.date(year, month, day)
        run_date = f"{solar_date} {hour:02d}:{minute:02d}:00"
        args = [msg, month, day, hour, minute]
        func = self.report_solar_birthday
        self.scheduler.add_job(func, "date", run_date=run_date, args=args)

    def report_lunar_birthday(self, msg, lunar_month, lunar_day, hour=8, minute=3):
        self.report_func(msg)

        if self.scheduler is None:
            return

        # add next year's schedule job
        now_lunar_date = self.get_now_lunar_date()
        self._add_lunar_schedule(
            msg, now_lunar_date.year + 1, lunar_month, lunar_day, hour, minute
        )

    def report_solar_birthday(self, msg, month, day, hour=8, minute=3):
        self.report_func(msg)

        if self.scheduler is None:
            return

        # add next year's schedule job
        today = datetime.date.today()
        next_year = today.year + 1
        self._add_solar_schedule(msg, next_year, month, day, hour, minute)

    def add_lunar_schedule(
        self,
        lunar_month,
        lunar_day,
        hour=8,
        minute=3,
        who="someone",
        greeting_words=None,
    ):
        if self.scheduler is None:
            logger.error("Scheduler is not set. Please set_scheduler() first.")
            return

        msg = self.generate_birthday_greeting(who, greeting_words)

        now_lunar_date = self.get_now_lunar_date()
        lunar_date = LunarDate(now_lunar_date.year, lunar_month, lunar_day)
        if lunar_date < LunarDate.today():
            lunar_date = LunarDate(now_lunar_date.year + 1, lunar_month, lunar_day)

        self._add_lunar_schedule(
            msg, lunar_date.year, lunar_month, lunar_day, hour, minute
        )

    def add_solar_schedule(
        self, month, day, hour=8, minute=3, who="someone", greeting_words=None
    ):
        if self.scheduler is None:
            logger.error("Scheduler is not set. Please set_scheduler() first.")
            return

        msg = self.generate_birthday_greeting(who, greeting_words)

        now = datetime.datetime.now()
        solar_date = datetime.date(now.year, month, day)
        if solar_date < now.date():
            solar_date = solar_date.replace(year=now.year + 1)

        self._add_solar_schedule(msg, solar_date.year, month, day, hour, minute)


class Deepseek(Notice):
    def __init__(self, keys_filepath="pten_keys.ini", **kwargs):
        super().__init__()
        self.keys = Keys(keys_filepath)
        sk_api_key = self.keys.get_key("notice", "deepseek_api_key")
        self.deepseek_client = OpenAI(
            api_key=sk_api_key,
            base_url="https://api.deepseek.com",
        )

    def get_completion(self, prompt):
        response = self.deepseek_client.chat.completions.create(
            model="deepseek-chat",
            messages=[
                {"role": "system", "content": "You are a helpful assistant"},
                {"role": "user", "content": prompt},
            ],
            stream=False,
        )

        return response.choices[0].message.content


class Weather(Notice):
    def __init__(self, keys_filepath="pten_keys.ini", **kwargs):
        super().__init__()
        self.keys = Keys(keys_filepath)
        self.api_key = self.keys.get_key("notice", "seniverse_api_key")
        self.cities = {}

    def add_city(self, city_name, city_code):
        self.cities[city_name] = city_code

    def get_weather(self, city_code):
        url = f"https://api.seniverse.com/v3/weather/now.json?key={self.api_key}&location={city_code}&language=zh-Hans&unit=c"

        response = requests.get(url)
        weather_json = json.loads(response.text)
        weather = weather_json["results"][0]["now"]

        return weather

    def get_city_weather(self, city_name, city_code):
        weather = self.get_weather(city_code)

        weather_text = weather.get("text")
        weather_temperature = weather.get("temperature")

        weather_str = f"{city_name}: {weather_text},\t 温度: {weather_temperature}度"

        return weather_str

    def report_weather(self):
        if not self.cities:
            logger.warning("No city added. Please add_city() first.")
            return
        weather_str = "今日天气："
        for city_name, city_code in self.cities.items():
            weather_str += "\n" + self.get_city_weather(city_name, city_code)

        self.report_func(weather_str)
